local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local GGL_Templates = GGL.Templates
local L = GGL.Locale

-- lua

-- wow

-- GGL
local MAIN_SOURCE_FLAG, SUB_SOURCE_FLAG = GGL.MAIN_SOURCE_FLAG, GGL.SUB_SOURCE_FLAG
local GetMoneyTextShort = GGL.GetMoneyTextShort
local GetMoneyText = GGL.GetMoneyText

-- Init
local BaseFrame, Templates = GGL.GUI.Statistics:RegisterFrameType("baseFrame", L["Statistics"])

local SortList = {
    ["items"] = {
        {L["Test"], function(private)

        end},
    },
    ["enemys"] = {
        {L["Item Value Summary"], function(private)
            local sortList = private.listContainer.children
            for i = 1,#sortList do
                sortList[i]:SetSortValue(sortList[i].list.fullItemValue)
            end
        end},
        {L["Item Count"], function(private)
            local sortList = private.listContainer.children
            for i = 1,#sortList do
                sortList[i]:SetSortValue(sortList[i].list.fullItemCount)
            end
        end},
        {L["Name"], function(private)
            local sortList = private.listContainer.children
            for i = 1,#sortList do
                sortList[i]:SetSortValue(sortList[i]._text)
            end
        end},
    },
}

local function Sort_DropDown_OnValueChanged(self, event, value)
    SortList[ self.obj.uName ][value][2](self.obj)
    self.obj.sortValue = value
    self.obj.listContainer:SortChildrenList()
end

local function SortOrder_DropDown_OnValueChanged(self, event, value)
    self.obj.sortOrder = value
    if value == 1 then
        self.obj.listContainer:SetDescending(false)
    elseif value == 2 then
        self.obj.listContainer:SetDescending(true)
    end
    Sort_DropDown_OnValueChanged(self, event, self.obj.sortValue or 1)
end

BaseFrame:SetDropDownList({
    -- build
    { "overview", L["Overview"], function(self, private, content)
        -- load buttons here
        self:AddNpcExportButton()
    end},

    { "items", L["Items"], function(self, private, content)
        --local listContainerBG = GGL.GUI:CreateBorderedFrame(content)
        local listContainer = AceGUI:Create("GoldGoblinLootGUIContainer-TabelView")
        listContainer.frame:ClearAllPoints()
        listContainer.frame:SetParent(content)
        listContainer:SetFullWidth(true)
        listContainer.frame:SetPoint("TOPLEFT", content, 2, -2)
        listContainer.frame:SetPoint("BOTTOMRIGHT", content, -2, 2)
        listContainer.frame:Show()
        --listContainerBG:SetPoint("TOPLEFT", listContainer.scrollframe, -5, 5)
        --listContainerBG:SetPoint("BOTTOMRIGHT", listContainer.scrollframe, 5, -5)
        private.listContainer = listContainer
    end},

    { "enemys", L["Enemys"], function(self, private, content)
        local orderDropdown = AceGUI:Create("Dropdown")
        orderDropdown:ClearAllPoints()
        orderDropdown.frame:SetParent(content)
        orderDropdown:SetCallback("OnValueChanged", SortOrder_DropDown_OnValueChanged)
        orderDropdown.frame:SetPoint("TOPRIGHT", content, -2, -2)
        orderDropdown.frame:SetPoint("RIGHT", content, -55, -2)
        orderDropdown.frame:Show()
        orderDropdown.obj = private
        private.orderDropdown = orderDropdown

        -- content
        orderDropdown:SetList({L["Highest"], L["Lowest"]}, {1,2})

        local dropdown = AceGUI:Create("Dropdown")
        dropdown:ClearAllPoints()
        dropdown.frame:SetParent(content)
        dropdown:SetCallback("OnValueChanged", Sort_DropDown_OnValueChanged)
        dropdown.frame:SetPoint("TOPLEFT", content, 2, -2)
        dropdown.frame:SetPoint("RIGHT", orderDropdown.frame, "LEFT", -2, -2)
        dropdown.frame:Show()
        dropdown.obj = private
        private.dropdown = dropdown

        -- create sorterDD
        local fillList, sortList = {}, {}
        for i = 1, #SortList[private.uName] do
            sortList[i] = i
            fillList[i] = SortList[private.uName][i][1]
        end
        dropdown:SetList(fillList, sortList)

        --local listContainerBG = GGL.GUI:CreateBorderedFrame(content)
        local listContainer = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
        listContainer.frame:ClearAllPoints()
        listContainer.frame:SetParent(content)
        listContainer:SetFullWidth(true)
        listContainer:SetPoint("TOPLEFT", dropdown.frame, "BOTTOMLEFT", 0, -2)
        listContainer:SetPoint("BOTTOMRIGHT", content, -2, 2)
        listContainer:SetLayout("GGL-ListWithGap")
        --listContainerBG:SetPoint("TOPLEFT", listContainer.scrollframe, -5, 5)
        --listContainerBG:SetPoint("BOTTOMRIGHT", listContainer.scrollframe, 5, -5)
        private.listContainer = listContainer
    end},
})

BaseFrame:SetOnShowList({
    ["overview"] = function(self, private, content)

    end,

    ["items"] = function(self, private, content)
        if not self.statistics then return end

        local listContainer = private.listContainer
        listContainer:SetFreeScaleCell(2)

        local npcList = {}
        for k, v in pairs(GoldGoblinLootDataDB and GoldGoblinLootDataDB["npc_names"] or {}) do
            npcList[#npcList+1] = {k, v}
        end

        listContainer:SetContent({"id", "name"}, npcList)
    end,

    ["enemys"] = function(self, private, content)
        if not self.statistics then return end
        local listContainer = private.listContainer

        -- get overall npc data
        local npcData, npcCache = self.statistics:GetNPCData()

        for npcID, npc in pairs(npcData) do
            local bar = AceGUI:Create("GoldGoblinLootGUIWidget-QualityProgressBar")
            bar:SetFullWidth(true)
            bar:ShowValueText(true)
            bar:SetQualityList(npc)
            bar:SetText(npcCache[npcID] or "npc:"..npcID)
            bar:SetRightText(GetMoneyText(npc.fullItemValue))
            --bar:SetSortValue(npc.fullItemValue)
            bar:SetUserData("listGap", 2)
            listContainer:AddChild(bar)
        end

        private.npcData = npcData

        private.orderDropdown:SetValue(private.sortOrder or 1)
        SortOrder_DropDown_OnValueChanged(private.orderDropdown, nil, private.sortOrder or 1)

        private.dropdown:SetValue(private.sortValue or 1)
        Sort_DropDown_OnValueChanged(private.dropdown, nil, private.sortValue or 1)
    end,
})

BaseFrame:SetOnHideList({
    ["overview"] = function(self, private, content)

    end,

    ["items"] = function(self, private, content)
        private.listContainer:ReleaseChildren()
    end,

    ["enemys"] = function(self, private, content)
        private.listContainer:ReleaseChildren()
    end,
})