local AddonName, GGL = ...
local Templates = GGL.Templates

local ItemFilter, ItemFilterHelper = Templates:Register("ItemFilter")

local AceGUI = LibStub("AceGUI-3.0")

local VERSION = 1

-- lua
local tonumber, error = tonumber, error
local gmatch = string.gmatch
local pairs, concat = pairs, table.concat

-- wow
local GetItemClassInfo, GetItemSubClassInfo = GetItemClassInfo, GetItemSubClassInfo
local GetItemInfo, GetItemInfoInstant = GetItemInfo, GetItemInfoInstant
local GetPetInfoByItemID = C_PetJournal and C_PetJournal.GetPetInfoByItemID or false

local L = GGL.Locale
-- ##################
-- local
-- ##################
local ITEM_TYPE_FILTER, SUB_ITEM_TYPE_FILTER

-- starts with 0 not 1
if GGL.IS_RETAIL then
    ITEM_TYPE_FILTER = {
        -- Obsolete
        [LE_ITEM_CLASS_REAGENT] = true,
        [LE_ITEM_CLASS_PROJECTILE] = true,
        [LE_ITEM_CLASS_QUIVER] = true,
        [LE_ITEM_CLASS_KEY] = true,
        [10] = true, -- Money(OBSOLETE)
        [14] = true, -- Permanent(OBSOLETE)
        -- Hide WoW Token
        [LE_ITEM_CLASS_WOW_TOKEN] = true, -- Permanent(OBSOLETE)
    }
    SUB_ITEM_TYPE_FILTER = {
        [LE_ITEM_CLASS_CONSUMABLE] = { -- Projectile
            [4] = true, -- Scroll (OBSOLETE)
            [6] = true, -- Item Enhancement (OBSOLETE)
        },
        [LE_ITEM_CLASS_TRADEGOODS] = { -- Tradeskill
            [0] = true, -- Trade Goods (OBSOLETE)
            [2] = true, -- Explosives (OBSOLETE)
            [3] = true, -- Devices (OBSOLETE)
            [13] = true, -- Materials (OBSOLETE)
            [14] = true, -- Item Enchantment (OBSOLETE)
            [15] = true, -- Weapon Enchantment - Obsolete
            [17] = true, -- Explosives and Devices (OBSOLETE)
        },
    }
elseif GGL.IS_CLASSIC then
    ITEM_TYPE_FILTER = {
        -- Obsolete
        [10] = true, -- Money(OBSOLETE)
        [14] = true, -- Permanent(OBSOLETE)

        [3] = true, -- Jewelry(OBSOLETE)
        [8] = true, -- Generic(OBSOLETE)
        -- Hide WoW Token
        [LE_ITEM_CLASS_WOW_TOKEN] = true, -- Permanent(OBSOLETE)
    }
    SUB_ITEM_TYPE_FILTER = {

    }
elseif GGL.IS_BCC or GGL.IS_WRATH then
    ITEM_TYPE_FILTER = {
        -- Obsolete
        [10] = true, -- Money(OBSOLETE)
        [14] = true, -- Permanent(OBSOLETE)

        [8] = true, -- Generic(OBSOLETE)
        -- Hide WoW Token
        [LE_ITEM_CLASS_WOW_TOKEN] = true, -- Permanent(OBSOLETE)
    }
    SUB_ITEM_TYPE_FILTER = {

    }
end

local ADD_SET_STRING = format("|A:%s:14:14:0:0|a |cff00ff00%s|r", GGL.ATLAS_TEXTURE["+"], L["Add new"])
local CONFIG_STRING = format("|A:%s:16:16:0:0|a %s", GGL.ATLAS_TEXTURE["config"], L["Config"])
-- ##################
-- static
-- ##################
local Static = {}
Templates.ItemFilter = Static

function Static:GetFilterIgnores()
    return ITEM_TYPE_FILTER, SUB_ITEM_TYPE_FILTER
end

-- ##################
-- template
-- ##################
function ItemFilter:Constructor(setKey, isSettingString)
    if isSettingString then
        self:LoadSetSettings(setKey)
    else
        self:LoadSet(setKey)
    end
end

function ItemFilter:LoadSet(setKey)
    local db = GGL.db.Config.Sets[GGL.SET_ID_LIST.Item]
    self.setKey = setKey
    if db and db[setKey] then
        local pass, db = GGL:Deserialize(db[setKey].setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
end

function ItemFilter:LoadSetSettings(setting)
    if setting then
        local pass, db = GGL:Deserialize(setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
end

function ItemFilter:ReloadSet()
    if self.setKey then
        self:LoadSet(self.setKey)
    end
end

function ItemFilter:SetFilterDB(db)
    self.db = db or {}
end

function ItemFilter:IsValidItem(itemID, itemString)
    local _, itemType, itemSubType, itemEquipLoc, icon, itemClassID, itemSubClassID = GetItemInfoInstant(itemID)
    local db = self.db
    --- first check instant things
    -- global ignore
    if ITEM_TYPE_FILTER[itemClassID] or (SUB_ITEM_TYPE_FILTER[itemClassID] and SUB_ITEM_TYPE_FILTER[itemClassID][itemSubClassID]) then
        return false
    end
    -- check itemClass
    if db[itemClassID] == true or ( db[itemClassID] and db[itemClassID][itemSubClassID] == true ) then
        return false
    end
    -- check if pet
    if GetPetInfoByItemID and itemClassID == 15 and itemSubClassID == 2 then
        local _, _, _, _, _, _, _, _, _, _, _, _, speciesID = GetPetInfoByItemID(itemID)
        if speciesID then
            return true
        end
    end
    --- not instant, hope its cached?
    local itemName, _, itemQuality, _, _, _, _, _, _, _, _, _, _, bindType = GetItemInfo(itemString or itemID)
    -- not cached or no valid itemID
    if not itemName then
        return nil
    end

    -- check soulbound
    if bindType == 1 and db["s"] then
        return false
    end

    -- check quality filter
    if db["q"..itemClassID] then
        if db["q"..itemClassID][itemQuality] then
            return false
        end
    elseif db["q"] and db["q"][itemQuality] then
        return false
    end

    -- all checks OK
    return true
end